package de.intevation.flys.collections;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.xml.xpath.XPathConstants;

import org.apache.log4j.Logger;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import de.intevation.artifacts.ArtifactDatabase;
import de.intevation.artifacts.ArtifactDatabaseException;
import de.intevation.artifacts.ArtifactNamespaceContext;
import de.intevation.artifacts.CallMeta;

import de.intevation.artifactdatabase.state.DefaultOutput;
import de.intevation.artifactdatabase.state.Facet;
import de.intevation.artifactdatabase.state.Output;

import de.intevation.artifacts.common.utils.XMLUtils;

import de.intevation.flys.artifacts.model.ManagedFacet;


public class AttributeParser {

    /** Constant XPath that points to the outputmodes of an artifact.*/
    public static final String XPATH_ARTIFACT_OUTPUTMODES =
        "/art:output";


    private static Logger logger = Logger.getLogger(AttributeParser.class);


    protected Map<String, Output> outs;


    public AttributeParser() {
        this.outs = new HashMap<String, Output>();
    }


    public void parse(Document doc) {
        logger.debug("AttributeParser.parse");

        NodeList outs = (NodeList) XMLUtils.xpath(
            doc,
            XPATH_ARTIFACT_OUTPUTMODES,
            XPathConstants.NODESET,
            ArtifactNamespaceContext.INSTANCE);

        int num = outs != null ? outs.getLength() : 0;

        logger.debug("Attribute has " + num + " outputs.");

        for (int i = 0; i < num; i++) {
            Node out = outs.item(i);

            parseOutput(out);
        }
    }


    public Map<String, Output> getOuts() {
        return outs;
    }


    protected void addItem(String out, ManagedFacet item) {
        Output o = outs.get(out);

        if (o != null) {
            o.addFacet(item);
        }
    }


    protected void parseOutput(Node out) {
        String name = XMLUtils.xpathString(
            out, "@name", ArtifactNamespaceContext.INSTANCE);

        if (outs.get(name) == null) {
            logger.debug("Create new output: " + name);

            Output o = new DefaultOutput(name, null, null);
            outs.put(name, o);
        }

        parseItems(out, name);
    }


    protected void parseItems(Node out, String outname) {
        NodeList themes = (NodeList) XMLUtils.xpath(
            out, "art:theme",
            XPathConstants.NODESET,
            ArtifactNamespaceContext.INSTANCE);

        int num = themes != null ? themes.getLength() : 0;

        logger.debug("Output has " + num + " themes.");

        for (int i = 0; i < num; i++) {
            Node theme = themes.item(i);

            String name = XMLUtils.xpathString(
                theme, "@facet", ArtifactNamespaceContext.INSTANCE);

            String uuid = XMLUtils.xpathString(
                theme, "@artifact", ArtifactNamespaceContext.INSTANCE);

            String pos = XMLUtils.xpathString(
                theme, "@pos", ArtifactNamespaceContext.INSTANCE);

            String active = XMLUtils.xpathString(
                theme, "@active", ArtifactNamespaceContext.INSTANCE);

            ManagedFacet item = new ManagedFacet(
                name, "", uuid,
                Integer.parseInt(pos),
                Integer.parseInt(active));

            addItem(outname, item);
        }
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
